"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CloverConnector = void 0;
var sdk = require("remote-pay-cloud-api");
var CardEntryMethods_1 = require("./CardEntryMethods");
var CloverConnectorBroadcaster_1 = require("./CloverConnectorBroadcaster");
var CloverDeviceFactory_1 = require("./device/CloverDeviceFactory");
var Logger_1 = require("./util/Logger");
var Builder_1 = require("../../util/PayIntent/Builder");
/**
 * Clover Connector
 *
 * The clover connector implements the ICloverConnector interface. This is where
 * we define how the connector interacts with remote pay adapters.
 */
var CloverConnector = /** @class */ (function () {
    function CloverConnector(config) {
        // manual is not enabled by default
        this.cardEntryMethods = CardEntryMethods_1.CardEntryMethods.DEFAULT;
        // Create a logger
        this.logger = Logger_1.Logger.create();
        // List of listeners to broadcast notifications to
        this.broadcaster = new CloverConnectorBroadcaster_1.CloverConnectorBroadcaster();
        // Flag indicating whether the device is ready or not
        this.isReady = false;
        // Set the cancel input option
        CloverConnector.CANCEL_INPUT_OPTION = new sdk.remotemessage.InputOption();
        CloverConnector.CANCEL_INPUT_OPTION.setKeyPress(sdk.remotemessage.KeyPress.ESC);
        CloverConnector.CANCEL_INPUT_OPTION.setDescription("Cancel");
        // Try to load the configuration.
        if (config) {
            this.configuration = config;
        }
    }
    /**
     * Initialize the connector with a new config
     *
     * @param {CloverDeviceConfiguration} config - the configuration for the connector
     */
    CloverConnector.prototype.initialize = function (config) {
        this.configuration = config;
        this.deviceObserver = new CloverConnector.InnerDeviceObserver(this);
        // Get the device and subscribe to it.
        this.device = CloverDeviceFactory_1.CloverDeviceFactory.get(config);
        if (this.device) {
            this.device.subscribe(this.deviceObserver);
        }
    };
    CloverConnector.prototype.initializeConnection = function () {
        if (!this.device) {
            this.initialize(this.configuration);
        }
    };
    /**
     * Add new listener to receive broadcast notifications
     *
     * @param {sdk.remotepay.ICloverConnectorListener} connectorListener - the listener to add
     */
    CloverConnector.prototype.addCloverConnectorListener = function (connectorListener) {
        this.broadcaster.push(connectorListener);
    };
    /**
     * Remove a listener
     *
     * @param {sdk.remotepay.ICloverConnectorListener} connectorListener - the listener to remove
     */
    CloverConnector.prototype.removeCloverConnectorListener = function (connectorListener) {
        var indexOfListener = this.broadcaster.indexOf(connectorListener);
        if (indexOfListener != -1) {
            this.broadcaster.splice(indexOfListener, 1);
        }
    };
    CloverConnector.prototype.sale = function (request) {
        if (!this.device || !this.isReady) {
            this.deviceObserver.onFinishCancelSale(sdk.remotepay.ResponseCode.ERROR, "Device Connection Error", "In sale: SaleRequest - The Clover device is not connected.");
        }
        else if (request.getVaultedCard() && !this.merchantInfo.getSupportsVaultCards()) {
            this.deviceObserver.onFinishCancel_rmm(sdk.remotepay.ResponseCode.UNSUPPORTED, "Merchant Configuration Validation Error", "In sale: SaleRequest - Vault Card support is not enabled for the payment gateway. Original Request = " + request, CloverConnector.TxTypeRequestInfo.SALE_REQUEST);
        }
        else {
            if (request.getTipAmount() == null) {
                request.setTipAmount(0);
            }
            try {
                this.setupSaleRequest(request);
            }
            catch (e) {
                this.logger.debug("Error in sale", e);
                this.deviceObserver.onFinishCancel_rmm(sdk.remotepay.ResponseCode.ERROR, e, null, CloverConnector.TxTypeRequestInfo.SALE_REQUEST);
            }
        }
    };
    CloverConnector.prototype.setupPreauthRequest = function (request) {
        this.setupBaseTransactionRequest(request, new sdk.payments.TransactionSettings(), new Builder_1.PayIntent.Builder(), CloverConnector.TxTypeRequestInfo.PREAUTH_REQUEST);
    };
    CloverConnector.prototype.setupAuthRequest = function (request) {
        var transactionSettings = new sdk.payments.TransactionSettings();
        transactionSettings.setTipMode(sdk.payments.TipMode.ON_PAPER); // overriding TipMode, since it's an Auth request
        this.setupTransactionRequest(request, transactionSettings, new Builder_1.PayIntent.Builder(), CloverConnector.TxTypeRequestInfo.AUTH_REQUEST);
    };
    CloverConnector.prototype.setupSaleRequest = function (request) {
        var transactionSettings = new sdk.payments.TransactionSettings();
        if (request.getTipMode() != null) {
            transactionSettings.setTipMode(request.getTipMode());
        }
        var builder = new Builder_1.PayIntent.Builder();
        if (request.getTipAmount() != null) {
            builder.setTipAmount(request.getTipAmount());
        }
        this.setupTransactionRequest(request, transactionSettings, builder, CloverConnector.TxTypeRequestInfo.SALE_REQUEST);
    };
    CloverConnector.prototype.setupTransactionRequest = function (request, transactionSettings, builder, paymentRequestType) {
        if (request.getSignatureEntryLocation() != null) {
            transactionSettings.setSignatureEntryLocation(request.getSignatureEntryLocation());
        }
        if (request.getSignatureThreshold() != null) {
            transactionSettings.setSignatureThreshold(request.getSignatureThreshold());
        }
        if (request.getAutoAcceptSignature() != null) {
            transactionSettings.setAutoAcceptSignature(request.getAutoAcceptSignature());
        }
        if (request.getDisableCashback() != null) {
            transactionSettings.setDisableCashBack(request.getDisableCashback());
        }
        if (request.getAllowOfflinePayment() != null) {
            transactionSettings.setAllowOfflinePayment(request.getAllowOfflinePayment());
        }
        if (request.getForceOfflinePayment() != null) {
            transactionSettings.setForceOfflinePayment(request.getForceOfflinePayment());
        }
        if (request.getApproveOfflinePaymentWithoutPrompt() != null) {
            transactionSettings.setApproveOfflinePaymentWithoutPrompt(request.getApproveOfflinePaymentWithoutPrompt());
        }
        if (request.getTaxAmount() != null) {
            builder.setTaxAmount(request.getTaxAmount());
        }
        if (request.getTippableAmount() != null) {
            transactionSettings.setTippableAmount(request.getTippableAmount());
        }
        if (request.getTipSuggestions() != null) {
            transactionSettings.setTipSuggestions(request.getTipSuggestions());
        }
        this.setupBaseTransactionRequest(request, transactionSettings, builder, paymentRequestType);
    };
    CloverConnector.prototype.setupBaseTransactionRequest = function (request, transactionSettings, builder, paymentRequestType) {
        if (this.device != null && this.isReady) {
            builder.setTransactionType(request.getType());
            builder.setAmount(request.getAmount());
            builder.setVaultedCard(request.getVaultedCard());
            builder.setExternalPaymentId(request.getExternalId() != null ? request.getExternalId().trim() : null);
            builder.setRequiresRemoteConfirmation(true);
            if (request.getCardNotPresent() != null) {
                builder.setCardNotPresent(request.getCardNotPresent());
            }
            transactionSettings.setCardEntryMethods(request.getCardEntryMethods() != null ? request.getCardEntryMethods() : this.cardEntryMethods);
            if (request.getDisablePrinting() != null) {
                transactionSettings.setCloverShouldHandleReceipts(!request.getDisablePrinting());
            }
            if (request.getDisableRestartTransactionOnFail() != null) {
                transactionSettings.setDisableRestartTransactionOnFailure(request.getDisableRestartTransactionOnFail());
            }
            if (request.getDisableReceiptSelection() != null) {
                transactionSettings.setDisableReceiptSelection(request.getDisableReceiptSelection());
            }
            if (request.getDisableDuplicateChecking() != null) {
                transactionSettings.setDisableDuplicateCheck(request.getDisableDuplicateChecking());
            }
            if (request.getAutoAcceptPaymentConfirmations() != null) {
                transactionSettings.setAutoAcceptPaymentConfirmations(request.getAutoAcceptPaymentConfirmations());
            }
            if (request.getRegionalExtras() != null) {
                transactionSettings.setRegionalExtras(request.getRegionalExtras());
            }
            if (request.getExtras() != null) {
                builder.setPassThroughValues(request.getExtras());
            }
            if (request.getExternalReferenceId() != null) {
                builder.setExternalReferenceId(request.getExternalReferenceId());
            }
            if (request.getPresentQrcOnly() != null) {
                builder.setIsPresentQrcOnly(request.getPresentQrcOnly());
            }
            builder.setTransactionSettings(transactionSettings);
            var payIntent = builder.build();
            this.device.doTxStart(payIntent, null, paymentRequestType);
        }
    };
    CloverConnector.prototype.notifyDeviceNotConnected = function (message) {
        this.notifyDeviceError(sdk.remotepay.ErrorType.COMMUNICATION, sdk.remotepay.DeviceErrorEventCode.NotConnected, null, message + ": Device is not connected.");
    };
    CloverConnector.prototype.notifyInvalidData = function (message) {
        this.notifyDeviceError(sdk.remotepay.ErrorType.VALIDATION, sdk.remotepay.DeviceErrorEventCode.InvalidParam, null, message);
    };
    CloverConnector.prototype.notifyDeviceError = function (errorType, errorCode, cause, message) {
        var deviceErrorEvent = new sdk.remotepay.CloverDeviceErrorEvent();
        deviceErrorEvent.setType(errorType);
        deviceErrorEvent.setCode(errorCode);
        deviceErrorEvent.setCause(cause);
        deviceErrorEvent.setMessage(message);
        this.broadcaster.notifyOnDeviceError(deviceErrorEvent);
    };
    CloverConnector.prototype.acceptSignature = function (request) {
        var logLocation = "In acceptSignature";
        if (!this.device || !this.isReady) {
            this.notifyDeviceNotConnected(logLocation);
        }
        else if (request == null) {
            this.notifyInvalidData(logLocation + ": VerifySignatureRequest cannot be null.");
        }
        else if (request.getPayment() == null || request.getPayment().getId() == null) {
            this.notifyInvalidData(logLocation + ": VerifySignatureRequest. Payment must have an ID.");
        }
        else {
            this.device.doSignatureVerified(request.getPayment(), true);
        }
    };
    CloverConnector.prototype.rejectSignature = function (request) {
        var logLocation = "In rejectSignature";
        if (!this.device || !this.isReady) {
            this.notifyDeviceNotConnected(logLocation);
        }
        else if (request == null) {
            this.notifyInvalidData(logLocation + ": VerifySignatureRequest cannot be null.");
        }
        else if (request.getPayment() == null || request.getPayment().getId() == null) {
            this.notifyInvalidData(logLocation + ": VerifySignatureRequest. Payment must have an ID.");
        }
        else {
            this.device.doSignatureVerified(request.getPayment(), false);
        }
    };
    CloverConnector.prototype.acceptPayment = function (payment) {
        var logLocation = "In acceptPayment";
        if (!this.device || !this.isReady) {
            this.notifyDeviceNotConnected(logLocation);
        }
        else if (payment == null) {
            this.notifyInvalidData(logLocation + ": Payment cannot be null.");
        }
        else if (payment.getId() == null) {
            this.notifyInvalidData(logLocation + ": Payment must have an ID.");
        }
        else {
            this.device.doAcceptPayment(payment);
        }
    };
    CloverConnector.prototype.rejectPayment = function (payment, challenge) {
        var logLocation = "In rejectPayment";
        if (!this.device || !this.isReady) {
            this.notifyDeviceNotConnected(logLocation);
        }
        else if (payment == null) {
            this.notifyInvalidData(logLocation + ": Payment cannot be null.");
        }
        else if (payment.getId() == null) {
            this.notifyInvalidData(logLocation + ": Payment must have an ID.");
        }
        else if (challenge == null) {
            this.notifyInvalidData(logLocation + ": Challenge cannot be null.");
        }
        else {
            this.device.doRejectPayment(payment, challenge);
        }
    };
    CloverConnector.prototype.auth = function (request) {
        if (!this.device || !this.isReady) {
            this.deviceObserver.onFinishCancelAuth(sdk.remotepay.ResponseCode.ERROR, "Device connection Error", "In auth: Auth Request - The Clover device is not connected.");
        }
        else if (!this.merchantInfo.getSupportsAuths()) {
            this.deviceObserver.onFinishCancelAuth(sdk.remotepay.ResponseCode.UNSUPPORTED, "Merchant Configuration Validation Error", "In auth: AuthRequest - Auths are not enabled for the payment gateway. Original Request = " + request);
        }
        else if (request.getVaultedCard() && !this.merchantInfo.getSupportsVaultCards()) {
            this.deviceObserver.onFinishCancel_rmm(sdk.remotepay.ResponseCode.UNSUPPORTED, "Merchant Configuration Validation Error", "In auth: AuthRequest - Vault Card support is not enabled for the payment gateway. Original Request = " + request, CloverConnector.TxTypeRequestInfo.AUTH_REQUEST);
        }
        else {
            try {
                this.setupAuthRequest(request);
            }
            catch (e) {
                this.logger.debug("Error in auth", e);
                this.deviceObserver.onFinishCancel_rmm(sdk.remotepay.ResponseCode.ERROR, e, null, CloverConnector.TxTypeRequestInfo.AUTH_REQUEST);
            }
        }
    };
    CloverConnector.prototype.preAuth = function (request) {
        if (!this.device || !this.isReady) {
            this.deviceObserver.onFinishCancelPreAuth(sdk.remotepay.ResponseCode.ERROR, "Device connection Error", "In preAuth: PreAuthRequest - The Clover device is not connected.");
        }
        else if (!this.merchantInfo.getSupportsPreAuths()) {
            this.deviceObserver.onFinishCancelPreAuth(sdk.remotepay.ResponseCode.UNSUPPORTED, "Merchant Configuration Validation Error", "In preAuth: PreAuthRequest - " +
                "PreAuths are not enabled for the payment gateway. Original Request = " + request);
        }
        else if (request.getVaultedCard() && !this.merchantInfo.getSupportsVaultCards()) {
            this.deviceObserver.onFinishCancel_rmm(sdk.remotepay.ResponseCode.UNSUPPORTED, "Merchant Configuration Validation Error", "In preAuth: PreAuthRequest - " +
                "Vault Card support is not enabled for the payment gateway. Original Request = " + request, CloverConnector.TxTypeRequestInfo.PREAUTH_REQUEST);
        }
        else {
            try {
                this.setupPreauthRequest(request);
            }
            catch (e) {
                this.logger.debug("Error in preAuth", e);
                this.deviceObserver.onFinishCancel_rmm(sdk.remotepay.ResponseCode.ERROR, e, null, CloverConnector.TxTypeRequestInfo.PREAUTH_REQUEST);
            }
        }
    };
    CloverConnector.prototype.capturePreAuth = function (request) {
        if (!this.device || !this.isReady) {
            this.deviceObserver.onCapturePreAuthError(sdk.remotepay.ResponseCode.ERROR, "Device connection Error", "In capturePreAuth: CapturePreAuth - The Clover device is not connected.", request.getPaymentId());
        }
        else if (!this.merchantInfo.getSupportsPreAuths()) {
            this.deviceObserver.onCapturePreAuthError(sdk.remotepay.ResponseCode.UNSUPPORTED, "Merchant Configuration Validation Error", "In capturePreAuth: PreAuth Captures are not enabled for the payment gateway. Original Request = " + request, request.getPaymentId());
        }
        else {
            try {
                this.device.doCaptureAuth(request.getPaymentId(), request.getAmount(), request.getTipAmount());
            }
            catch (e) {
                var response = new sdk.remotepay.CapturePreAuthResponse();
                CloverConnector.populateBaseResponse(response, false, sdk.remotepay.ResponseCode.UNSUPPORTED, "Pre Auths unsupported", "The currently configured merchant gateway does not support Capture Auth requests.");
                this.broadcaster.notifyOnCapturePreAuth(response);
            }
        }
    };
    CloverConnector.prototype.incrementPreAuth = function (request) {
        if (!this.device || !this.isReady) {
            this.deviceObserver.onIncrementPreAuthError(sdk.remotepay.ResponseCode.ERROR, "Device connection Error", "In incrementPreAuth: IncrementPreAuth - The Clover device is not connected.");
        }
        else if (!this.merchantInfo.getSupportsPreAuths()) {
            this.deviceObserver.onIncrementPreAuthError(sdk.remotepay.ResponseCode.UNSUPPORTED, "Merchant Configuration Validation Error", "In incrementPreAuth: Pre Auths are not enabled for the payment gateway. Original Request = " + request);
        }
        else {
            try {
                this.device.doIncrementPreAuth(request.getPaymentId(), request.getAmount());
            }
            catch (e) {
                var response = new sdk.remotepay.IncrementPreAuthResponse();
                CloverConnector.populateBaseResponse(response, false, sdk.remotepay.ResponseCode.UNSUPPORTED, "Pre Auths unsupported", "The currently configured merchant gateway does not support Incremental Pre Auth requests.");
                this.broadcaster.notifyOnIncrementPreAuthResponse(response);
            }
        }
    };
    CloverConnector.prototype.tipAdjustAuth = function (request) {
        var tarm = new sdk.remotemessage.TipAdjustResponseMessage();
        if (!this.device || !this.isReady) {
            tarm.setReason("Device connection Error");
            tarm.setMessage("In tipAdjustAuth: TipAdjustAuthRequest - The Clover device is not connected.");
            this.deviceObserver.onAuthTipAdjusted(tarm, sdk.remotepay.ResponseCode.ERROR);
        }
        else if (!this.merchantInfo.getSupportsTipAdjust()) {
            tarm.setReason("Merchant Configuration Validation Error");
            tarm.setMessage("In tipAdjustAuth: TipAdjustAuthRequest - Tip Adjustments are not enabled for the payment gateway. Original Request = " + request);
            this.deviceObserver.onAuthTipAdjusted(tarm, sdk.remotepay.ResponseCode.UNSUPPORTED);
        }
        else {
            this.device.doTipAdjustAuth(request.getOrderId(), request.getPaymentId(), request.getTipAmount());
        }
    };
    CloverConnector.prototype.vaultCard = function (cardEntryMethods) {
        var shortCircuitVaultCardResponseMessage = new sdk.remotemessage.VaultCardResponseMessage();
        shortCircuitVaultCardResponseMessage.setCard(null);
        if (!this.device || !this.isReady) {
            shortCircuitVaultCardResponseMessage.setReason("Device connection Error");
            this.deviceObserver.onVaultCardResponse(shortCircuitVaultCardResponseMessage, sdk.remotepay.ResponseCode.ERROR, "In vaultCard: The Clover device is not connected.");
        }
        else if (!this.merchantInfo.getSupportsVaultCards()) {
            shortCircuitVaultCardResponseMessage.setReason("Merchant Configuration Validation Error");
            this.deviceObserver.onVaultCardResponse(shortCircuitVaultCardResponseMessage, sdk.remotepay.ResponseCode.UNSUPPORTED, "In vaultCard: VaultCard/Payment Tokens are not enabled for the payment gateway.");
        }
        else {
            this.device.doVaultCard(cardEntryMethods ? cardEntryMethods : this.getCardEntryMethods());
        }
    };
    CloverConnector.prototype.voidPayment = function (request) {
        if (!this.device || !this.isReady) {
            this.deviceObserver.onPaymentVoided_responseCode(sdk.remotepay.ResponseCode.ERROR, "Device connection Error", "In voidPayment: VoidPaymentRequest - The Clover device is not connected.");
        }
        else {
            var payment = new sdk.payments.Payment();
            payment.setId(request.getPaymentId());
            payment.setOrder(new sdk.base.Reference());
            payment.getOrder().setId(request.getOrderId());
            payment.setEmployee(new sdk.base.Reference());
            payment.getEmployee().setId(request.getEmployeeId());
            var reason = sdk.order.VoidReason[request.getVoidReason()];
            this.device.doVoidPayment(payment, reason, request.getDisablePrinting(), request.getDisableReceiptSelection(), request.getExtras());
        }
    };
    CloverConnector.prototype.refundPayment = function (request) {
        if (!this.device || !this.isReady) {
            this.deviceObserver.onFinishCancel_rmm(sdk.remotepay.ResponseCode.CANCEL, "Device Connection Error", "In refundPayment: RefundPaymentRequest - The Clover device is not connected.", CloverConnector.TxTypeRequestInfo.REFUND_REQUEST);
        }
        else {
            this.device.doPaymentRefundByRequest(request);
        }
    };
    CloverConnector.prototype.voidPaymentRefund = function (request) {
        this.deviceObserver.onPaymentRefundVoided(null, sdk.remotepay.ResponseCode.ERROR, "Cannot do Void Payment Refund", "This version of the SDK does not support voidPaymentRefunds");
    };
    CloverConnector.prototype.manualRefund = function (request) {
        var transactionSettings = new sdk.payments.TransactionSettings();
        if (!this.device || !this.isReady) {
            this.deviceObserver.onFinishCancelManualRefund(sdk.remotepay.ResponseCode.ERROR, "Device connection Error", "In manualRefund: ManualRefundRequest - The Clover device is not connected.");
        }
        else if (!this.merchantInfo.getSupportsManualRefunds()) {
            this.deviceObserver.onFinishCancelManualRefund(sdk.remotepay.ResponseCode.UNSUPPORTED, "Merchant Configuration Validation Error", "In manualRefund: ManualRefundRequest - Manual Refunds are not enabled for the payment gateway. Original Request = " + request);
        }
        else if (request.getVaultedCard() && !this.merchantInfo.getSupportsVaultCards()) {
            this.deviceObserver.onFinishCancel_rmm(sdk.remotepay.ResponseCode.UNSUPPORTED, "Merchant Configuration Validation Error", "In manualRefund: ManualRefundRequest - Vault Card support is not enabled for the payment gateway. Original Request = " + request, CloverConnector.TxTypeRequestInfo.CREDIT_REQUEST);
        }
        else {
            var builder = new Builder_1.PayIntent.Builder();
            builder.setAmount(-Math.abs(request.getAmount()))
                .setTransactionType(sdk.remotepay.TransactionType.CREDIT)
                .setVaultedCard(request.getVaultedCard())
                .setRequiresRemoteConfirmation(true)
                .setExternalPaymentId(request.getExternalId());
            transactionSettings.setCardEntryMethods(request.getCardEntryMethods() ? request.getCardEntryMethods() : this.cardEntryMethods);
            if (request.getDisablePrinting()) {
                transactionSettings.setCloverShouldHandleReceipts(!request.getDisablePrinting());
            }
            if (request.getDisableRestartTransactionOnFail()) {
                transactionSettings.setDisableRestartTransactionOnFailure(request.getDisableRestartTransactionOnFail());
            }
            if (request.getDisableReceiptSelection()) {
                transactionSettings.setDisableReceiptSelection(request.getDisableReceiptSelection());
            }
            if (request.getExtras() != null) {
                builder.setPassThroughValues(request.getExtras());
            }
            builder.setTransactionSettings(transactionSettings);
            var payIntent = builder.build();
            this.device.doTxStart(payIntent, null, CloverConnector.TxTypeRequestInfo.CREDIT_REQUEST);
        }
    };
    CloverConnector.prototype.retrievePendingPayments = function () {
        if (!this.device || !this.isReady) {
            this.deviceObserver.onPendingPaymentsResponse(false, null, "Device connection Error", "In retrievePendingPayments: The Clover device is not connected.");
        }
        else {
            this.device.doRetrievePendingPayments();
        }
    };
    CloverConnector.prototype.readCardData = function (request) {
        if (!this.device || !this.isReady) {
            this.deviceObserver.onReadCardDataResponse(sdk.remotepay.ResponseCode.ERROR, "Device connection Error", "In readCardData: The Clover device is not connected.");
        }
        else if (request == null) {
            this.deviceObserver.onReadCardDataResponse(sdk.remotepay.ResponseCode.FAIL, "Invalid Argument.", "In readCardData: ReadCardDataRequest - The request that was passed in for processing is null.");
        }
        else {
            // create pay intent...
            var builder = new Builder_1.PayIntent.Builder();
            builder.setTransactionType(sdk.remotepay.TransactionType.DATA);
            builder.setCardEntryMethods(request.getCardEntryMethods() ? request.getCardEntryMethods() : this.cardEntryMethods);
            builder.setForceSwipePinEntry(request.getIsForceSwipePinEntry());
            var pi = builder.build();
            this.device.doReadCardData(pi);
        }
    };
    CloverConnector.prototype.sendMessageToActivity = function (request) {
        if (!this.device || !this.isReady) {
            this.notifyDeviceNotConnected("In sendMessageToActivity");
        }
        else if (request == null) {
            this.notifyInvalidData("In sendMessageToActivity: Invalid argument. Null is not allowed.");
        }
        else {
            this.device.doSendMessageToActivity(request.getAction(), request.getPayload());
        }
    };
    CloverConnector.prototype.retrievePayment = function (request) {
        if (!this.device || !this.isReady) {
            this.notifyDeviceNotConnected("In retrievePayment");
        }
        else {
            this.device.doRetrievePayment(request.getExternalPaymentId());
        }
    };
    CloverConnector.prototype.registerForCustomerProvidedData = function (request) {
        if (!this.device || !this.isReady) {
            this.notifyDeviceNotConnected("In registerForCustomerProvidedData");
        }
        else {
            var dataProviderConfigs = request.getConfigurations();
            var configurations = this.convertConfigurations(dataProviderConfigs);
            console.log(configurations);
            this.device.doRegisterForCustomerProvidedData(configurations);
        }
    };
    CloverConnector.prototype.convertConfigurations = function (dataProviderConfigs) {
        var _this = this;
        // this is here so we have a buffer between the public api and the private one.
        var configs = new Array();
        if (dataProviderConfigs != null) {
            dataProviderConfigs.forEach(function (dataProviderConfig) { return configs.push(_this.convertConfiguration(dataProviderConfig)); });
        }
        return configs;
    };
    CloverConnector.prototype.convertConfiguration = function (dataProviderConfig) {
        var loyaltyDataConfig = new sdk.loyalty.LoyaltyDataConfig();
        loyaltyDataConfig.setType(dataProviderConfig.getType());
        loyaltyDataConfig.setConfiguration(dataProviderConfig.getConfiguration());
        return loyaltyDataConfig;
    };
    CloverConnector.prototype.setCustomerInfo = function (request) {
        if (!this.device || !this.isReady) {
            this.notifyDeviceNotConnected("In setCustomerInfo: The Clover device is not connected.");
        }
        else {
            this.device.doSetCustomerInfo(request.getCustomerInfo());
        }
    };
    CloverConnector.prototype.retrievePrinters = function (request) {
        if (!this.device || !this.isReady) {
            this.notifyDeviceNotConnected("In retrievePrinters");
        }
        else if (!request) {
            this.notifyInvalidData("In retrievePrinters: Invalid argument. Null is not allowed.");
        }
        else {
            this.device.doRetrievePrinters(request.getCategory());
        }
    };
    CloverConnector.prototype.retrievePrintJobStatus = function (request) {
        if (!this.device || !this.isReady) {
            this.notifyDeviceNotConnected("In retrievePrintJobStatus");
        }
        else if (!request || !request.getPrintRequestId()) {
            this.notifyInvalidData("In retrievePrintJobStatus: Invalid argument. Null is not allowed.");
        }
        else {
            this.device.doRetrievePrintJobStatus(request.getPrintRequestId());
        }
    };
    CloverConnector.prototype.closeout = function (request) {
        if (!this.device || !this.isReady) {
            this.notifyDeviceNotConnected("In closeout");
        }
        else {
            this.device.doCloseout(request.getAllowOpenTabs(), request.getBatchId());
        }
    };
    CloverConnector.prototype.print = function (request) {
        if (!this.device || !this.isReady) {
            this.notifyDeviceNotConnected("In print");
        }
        else if (!request) {
            this.notifyInvalidData("In print: Invalid argument. Null is not allowed.");
        }
        else if (!this.validatePrintRequest(request)) {
            return;
        }
        else {
            var images = request.getImages();
            var urls = request.getImageUrls();
            if (images) {
                if (Array.isArray(images) && images.length > 1) {
                    this.notifyInvalidData("In print: Invalid argument. Only one image can be printed at a time in the current API.");
                }
                var singleOrArr = (Array.isArray(images) ? images[0] : images);
                this.device.doPrintImageObject(singleOrArr, request.getPrintRequestId(), request.getPrintDeviceId());
            }
            else if (request.getText()) {
                this.device.doPrintText(request.getText(), request.getPrintRequestId(), request.getPrintDeviceId());
            }
            else if (urls) {
                if (Array.isArray(urls) && urls.length > 1) {
                    this.notifyInvalidData("In print: Invalid argument. Only one imageUrl can be printed at a time in the current API.");
                }
                var singleOrArr = (Array.isArray(urls) ? urls[0] : urls);
                this.device.doPrintImageUrl(singleOrArr, request.getPrintRequestId(), request.getPrintDeviceId());
            }
            else {
                this.notifyInvalidData("In print: Invalid argument. PrintRequest element was not formatted correctly.");
            }
        }
    };
    CloverConnector.prototype.validatePrintRequest = function (request) {
        if (!request.getImages() && !request.getText() && !request.getImageUrls()) {
            this.notifyInvalidData("In validatePrintRequest: There are no items to print.");
            return false;
        }
        else if ((request.getImages() && request.getText()) ||
            (request.getImages() && request.getImageUrls()) ||
            (request.getText() && request.getImageUrls())) {
            this.notifyInvalidData("In validatePrintRequest: There are too may different kinds of items to print.  Can only have one.");
            return false;
        }
        else {
            return true;
        }
    };
    CloverConnector.prototype.showMessage = function (message) {
        if (!this.device || !this.isReady) {
            this.notifyDeviceNotConnected("In showMessage");
        }
        else if (message == null) {
            this.notifyInvalidData("In showMessage: Invalid argument.  Null is not allowed.");
        }
        else {
            this.device.doTerminalMessage(message);
        }
    };
    CloverConnector.prototype.sendDebugLog = function (message) {
        if (!this.device || !this.isReady) {
            this.notifyDeviceNotConnected("In sendDebugLog");
        }
        else if (message == null) {
            this.notifyInvalidData("In showMessage: Invalid argument.  Null is not allowed.");
        }
        else {
            this.device.doSendDebugLog(message);
        }
    };
    CloverConnector.prototype.showWelcomeScreen = function () {
        if (!this.device || !this.isReady) {
            this.notifyDeviceNotConnected("In showWelcomeScreen");
        }
        else {
            this.device.doShowWelcomeScreen();
        }
    };
    CloverConnector.prototype.showThankYouScreen = function () {
        if (!this.device || !this.isReady) {
            this.notifyDeviceNotConnected("In showThankYouScreen");
        }
        else {
            this.device.doShowThankYouScreen();
        }
    };
    CloverConnector.prototype.displayReceiptOptions = function (displayReceiptOptionsRequest) {
        if (!this.device || !this.isReady) {
            this.notifyDeviceNotConnected("In displayReceiptOptions");
        }
        else {
            this.device.doShowReceiptScreen(displayReceiptOptionsRequest.getOrderId(), displayReceiptOptionsRequest.getPaymentId(), displayReceiptOptionsRequest.getRefundId(), displayReceiptOptionsRequest.getCreditId(), displayReceiptOptionsRequest.getDisablePrinting());
        }
    };
    CloverConnector.prototype.openCashDrawer = function (request) {
        if (!this.device || !this.isReady) {
            this.notifyDeviceNotConnected("In openCashDrawer");
        }
        else if (!request) {
            this.notifyInvalidData("In openCashDrawer: Invalid argument. The request cannot be null.");
        }
        else if (typeof request === "string") {
            this.device.doOpenCashDrawer(request);
        }
        else {
            this.device.doOpenCashDrawer(request.getReason(), request.getDeviceId());
        }
    };
    CloverConnector.prototype.showDisplayOrder = function (order) {
        if (!this.device || !this.isReady) {
            this.notifyDeviceNotConnected("In showDisplayOrder");
        }
        else if (order == null) {
            this.notifyInvalidData("In showDisplayOrder: Invalid argument.  The order cannot be null.");
        }
        else {
            this.device.doOrderUpdate(order, null);
        }
    };
    CloverConnector.prototype.removeDisplayOrder = function (order) {
        if (!this.device || !this.isReady) {
            this.notifyDeviceNotConnected("In removeDisplayOrder");
        }
        else if (order == null) {
            this.notifyInvalidData("In removeDisplayOrder: Invalid argument.  The order cannot be null.");
        }
        else {
            var dao = new sdk.order.operation.OrderDeletedOperation();
            dao.setId(order.getId());
            this.device.doOrderUpdate(order, dao);
        }
    };
    // I spoke to Hammer about this, there are no plans to implement it.
    CloverConnector.prototype.discountAddedToDisplayOrder = function (discount, order) {
        this.notifyDeviceError(sdk.remotepay.ErrorType.EXCEPTION, sdk.remotepay.DeviceErrorEventCode.NotSupported, null, "discountAddedToDisplayOrder has not been implemented.");
    };
    // I spoke to Hammer about this, there are no plans to implement it.
    CloverConnector.prototype.discountRemovedFromDisplayOrder = function (discount, order) {
        this.notifyDeviceError(sdk.remotepay.ErrorType.EXCEPTION, sdk.remotepay.DeviceErrorEventCode.NotSupported, null, "discountRemovedFromDisplayOrder has not been implemented.");
    };
    // I spoke to Hammer about this, there are no plans to implement it.
    CloverConnector.prototype.lineItemAddedToDisplayOrder = function (lineItem, order) {
        this.notifyDeviceError(sdk.remotepay.ErrorType.EXCEPTION, sdk.remotepay.DeviceErrorEventCode.NotSupported, null, "lineItemAddedToDisplayOrder has not been implemented.");
    };
    // I spoke to Hammer about this, there are no plans to implement it.
    CloverConnector.prototype.lineItemRemovedFromDisplayOrder = function (lineItem, order) {
        this.notifyDeviceError(sdk.remotepay.ErrorType.EXCEPTION, sdk.remotepay.DeviceErrorEventCode.NotSupported, null, "lineItemRemovedFromDisplayOrder has not been implemented.");
    };
    CloverConnector.prototype.dispose = function () {
        this.broadcaster.clear();
        if (this.device) {
            this.device.dispose();
        }
    };
    CloverConnector.prototype.invokeInputOption = function (io) {
        if (!this.device || !this.isReady) {
            this.notifyDeviceNotConnected("In invokeInputOption");
        }
        else {
            this.device.doKeyPress(io.getKeyPress());
        }
    };
    CloverConnector.prototype.resetDevice = function () {
        if (!this.device || !this.isReady) {
            this.notifyDeviceNotConnected("In resetDevice");
        }
        else {
            this.device.doResetDevice();
        }
    };
    CloverConnector.prototype.retrieveDeviceStatus = function (request) {
        if (!this.device || !this.isReady) {
            this.notifyDeviceNotConnected("In retrieveDeviceStatus");
        }
        else {
            this.device.doRetrieveDeviceStatus(request);
        }
    };
    CloverConnector.prototype.getCardEntryMethods = function () {
        return this.cardEntryMethods;
    };
    CloverConnector.prototype.startCustomActivity = function (request) {
        if (!this.device || !this.isReady) {
            this.notifyDeviceNotConnected("In startCustomActivity");
        }
        else {
            this.device.doStartActivity(request.getAction(), request.getPayload(), request.getNonBlocking());
        }
    };
    CloverConnector.prototype.checkBalance = function (request) {
        if (!this.device || !this.isReady) {
            this.notifyDeviceNotConnected("In checkBalance");
        }
        else {
            this.device.doCheckBalance(request.getCardEntryMethods());
        }
    };
    CloverConnector.prototype.requestSignature = function (request) {
        if (!this.device || !this.isReady) {
            this.notifyDeviceNotConnected("In requestSignature");
        }
        else {
            this.device.doCollectSignature(request.getAcknowledgementMessage());
        }
    };
    CloverConnector.prototype.requestTip = function (request) {
        if (!this.device || !this.isReady) {
            this.notifyDeviceNotConnected("In requestTip");
        }
        else {
            this.device.doRequestTip(request.getTippableAmount(), request.getTipSuggestions());
        }
    };
    CloverConnector.populateBaseResponse = function (response, success, responseCode, reason, message) {
        response.setSuccess(success);
        response.setResult(responseCode);
        response.setReason(reason ? reason : responseCode ? responseCode.toString() : "No extended information provided.");
        response.setMessage(message ? message : "No extended information provided.");
        // See SSDK-549.  The setting of the response flags (isSale, etc.) is handled in each SDK.
        // The problem is that non-traditional payment responses (finishOk not called) may contain payment information.
        // To avoid the SDK of having to be aware of this we are inspecting the response and setting the flags if they
        // exist on the response and if a payment exists on the response.
        if (response["getPayment"]) {
            var payment = response["getPayment"]();
            if (payment) {
                this.setResponseFlags(response, payment);
            }
        }
    };
    CloverConnector.populatePaymentResponse = function (response, success, responseCode, payment, signature, reason, message) {
        CloverConnector.populateBaseResponse(response, success, responseCode, reason, message);
        response.setPayment(payment);
        response.setSignature(signature);
        this.setResponseFlags(response, payment);
    };
    CloverConnector.setResponseFlags = function (response, payment) {
        if (response && payment) {
            var cardTransaction = payment.getCardTransaction();
            if (cardTransaction) {
                var transactionType = cardTransaction.getType();
                if (response.setIsSale) {
                    response.setIsSale(sdk.payments.CardTransactionType.AUTH == transactionType &&
                        sdk.payments.Result.SUCCESS == payment.getResult());
                }
                if (response.setIsAuth) {
                    response.setIsAuth(sdk.payments.CardTransactionType.PREAUTH == transactionType &&
                        sdk.payments.Result.SUCCESS == payment.getResult());
                }
                if (response.setIsPreAuth) {
                    response.setIsPreAuth(sdk.payments.CardTransactionType.PREAUTH == transactionType &&
                        sdk.payments.Result.AUTH == payment.getResult());
                }
            }
        }
    };
    CloverConnector.MAX_PAYLOAD_SIZE = 10000000; // maximum size of the payload of a full message.  if the payload exceeds this, the message will not be sent.
    return CloverConnector;
}());
exports.CloverConnector = CloverConnector;
(function (CloverConnector) {
    var TxTypeRequestInfo = /** @class */ (function () {
        function TxTypeRequestInfo() {
        }
        TxTypeRequestInfo.SALE_REQUEST = "SALE";
        TxTypeRequestInfo.AUTH_REQUEST = "AUTH";
        TxTypeRequestInfo.PREAUTH_REQUEST = "PREAUTH";
        TxTypeRequestInfo.CREDIT_REQUEST = "CREDIT";
        TxTypeRequestInfo.REFUND_REQUEST = "REFUND";
        return TxTypeRequestInfo;
    }());
    CloverConnector.TxTypeRequestInfo = TxTypeRequestInfo;
    var InnerDeviceObserver = /** @class */ (function () {
        function InnerDeviceObserver(cc) {
            // Create a logger
            this.logger = Logger_1.Logger.create();
            this.cloverConnector = cc;
        }
        InnerDeviceObserver.prototype.onTxState = function (txState) {
        };
        InnerDeviceObserver.prototype.resultSuccessToResponseCode = function (success) {
            return this.resultStatusToResponseCode(success ? sdk.remotemessage.ResultStatus.SUCCESS : sdk.remotemessage.ResultStatus.FAIL);
        };
        InnerDeviceObserver.prototype.resultStatusToResponseCode = function (resultStatus) {
            return resultStatus == sdk.remotemessage.ResultStatus.SUCCESS ? sdk.remotepay.ResponseCode.SUCCESS : (resultStatus == sdk.remotemessage.ResultStatus.CANCEL ? sdk.remotepay.ResponseCode.CANCEL : sdk.remotepay.ResponseCode.FAIL);
        };
        InnerDeviceObserver.prototype.onTxStartResponse = function (txStartResponseMessage) {
            var reason = txStartResponseMessage.getReason();
            var result = txStartResponseMessage.getResult();
            if (result == sdk.remotemessage.TxStartResponseResult.SUCCESS)
                return;
            var duplicate = (result == sdk.remotemessage.TxStartResponseResult.DUPLICATE);
            var code = duplicate ? sdk.remotepay.ResponseCode.CANCEL : sdk.remotepay.ResponseCode.FAIL;
            var duplicateMessage = duplicate ? "The provided transaction id of " + txStartResponseMessage.getExternalPaymentId() + " has already been processed and cannot be resubmitted." : null;
            var messageToUse = duplicate ? duplicateMessage : txStartResponseMessage.getMessage();
            // Use the requestInfo if it exists, to determine the request type
            var requestInfo = txStartResponseMessage.getRequestInfo();
            if (requestInfo == TxTypeRequestInfo.PREAUTH_REQUEST) {
                var response = new sdk.remotepay.PreAuthResponse();
                CloverConnector.populateBaseResponse(response, false, code, reason, messageToUse);
                this.cloverConnector.broadcaster.notifyOnPreAuthResponse(response);
            }
            else if (requestInfo == TxTypeRequestInfo.AUTH_REQUEST) {
                var response = new sdk.remotepay.AuthResponse();
                CloverConnector.populateBaseResponse(response, false, code, reason, messageToUse);
                this.cloverConnector.broadcaster.notifyOnAuthResponse(response);
            }
            else if (requestInfo == TxTypeRequestInfo.SALE_REQUEST) {
                var response = new sdk.remotepay.SaleResponse();
                CloverConnector.populateBaseResponse(response, false, code, reason, messageToUse);
                this.cloverConnector.broadcaster.notifyOnSaleResponse(response);
            }
            else if (requestInfo == TxTypeRequestInfo.CREDIT_REQUEST) {
                var response = new sdk.remotepay.ManualRefundResponse();
                CloverConnector.populateBaseResponse(response, false, code, reason, messageToUse);
                this.cloverConnector.broadcaster.notifyOnManualRefundResponse(response);
            }
            else {
                this.logger.error("Could not determine request type. requestInfo = " + requestInfo);
            }
        };
        InnerDeviceObserver.prototype.onUiState = function (uiState, uiText, uiDirection, inputOptions) {
            var deviceEvent = new sdk.remotepay.CloverDeviceEvent();
            deviceEvent.setInputOptions(inputOptions);
            deviceEvent.setEventState(sdk.remotepay.DeviceEventState[uiState.toString()]);
            deviceEvent.setMessage(uiText);
            if (uiDirection == sdk.remotemessage.UiDirection.ENTER) {
                this.cloverConnector.broadcaster.notifyOnDeviceActivityStart(deviceEvent);
            }
            else if (uiDirection == sdk.remotemessage.UiDirection.EXIT) {
                this.cloverConnector.broadcaster.notifyOnDeviceActivityEnd(deviceEvent);
                if (uiState.toString() == sdk.remotepay.DeviceEventState.RECEIPT_OPTIONS.toString()) {
                    this.cloverConnector.device.doShowWelcomeScreen();
                }
            }
        };
        InnerDeviceObserver.prototype.onTipAdded = function (tip) {
            this.cloverConnector.broadcaster.notifyOnTipAdded(tip);
        };
        InnerDeviceObserver.prototype.onAuthTipAdjusted = function (tarm, responseCode) {
            var response = new sdk.remotepay.TipAdjustAuthResponse();
            var success = tarm.getSuccess();
            if (!responseCode) {
                responseCode = this.resultSuccessToResponseCode(success);
            }
            CloverConnector.populateBaseResponse(response, success, responseCode, tarm.getReason(), tarm.getMessage());
            response.setPaymentId(tarm.getPaymentId());
            response.setTipAmount(tarm.getAmount());
            this.cloverConnector.broadcaster.notifyOnTipAdjustAuthResponse(response);
        };
        InnerDeviceObserver.prototype.onCashbackSelected = function (cashbackAmount) {
            //TODO: For future use
        };
        InnerDeviceObserver.prototype.onPartialAuth = function (partialAmount) {
            //TODO: For future use
        };
        InnerDeviceObserver.prototype.onFinishOkPayment = function (payment, signature, requestInfo) {
            try {
                this.cloverConnector.device.doShowThankYouScreen(); //need to do this first, so Listener implementation can replace the screen as desired
                if (requestInfo == TxTypeRequestInfo.PREAUTH_REQUEST) {
                    var response = new sdk.remotepay.PreAuthResponse();
                    CloverConnector.populatePaymentResponse(response, true, sdk.remotepay.ResponseCode.SUCCESS, payment, signature);
                    this.cloverConnector.broadcaster.notifyOnPreAuthResponse(response);
                }
                else if (requestInfo == TxTypeRequestInfo.AUTH_REQUEST) {
                    var response = new sdk.remotepay.AuthResponse();
                    CloverConnector.populatePaymentResponse(response, true, sdk.remotepay.ResponseCode.SUCCESS, payment, signature);
                    this.cloverConnector.broadcaster.notifyOnAuthResponse(response);
                }
                else if (requestInfo == TxTypeRequestInfo.SALE_REQUEST) {
                    var response = new sdk.remotepay.SaleResponse();
                    CloverConnector.populatePaymentResponse(response, true, sdk.remotepay.ResponseCode.SUCCESS, payment, signature);
                    this.cloverConnector.broadcaster.notifyOnSaleResponse(response);
                }
                else {
                    this.logger.error("Failed to pair this response: " + payment);
                }
            }
            finally {
                // do nothing for now...
            }
        };
        InnerDeviceObserver.prototype.onFinishOkCredit = function (credit) {
            this.cloverConnector.device.doShowWelcomeScreen();
            var response = new sdk.remotepay.ManualRefundResponse();
            CloverConnector.populateBaseResponse(response, true, sdk.remotepay.ResponseCode.SUCCESS);
            response.setCredit(credit);
            this.cloverConnector.broadcaster.notifyOnManualRefundResponse(response);
        };
        InnerDeviceObserver.prototype.onFinishOkRefund = function (refund) {
            this.cloverConnector.device.doShowWelcomeScreen();
            // NOTE: these two lines can eventually be removed (once refunds have the orderRef populated correctly):
            var lastRefundResponse = this.lastPRR; //only needed for the order ID
            this.lastPRR = null;
            if (refund.getOrderRef() != null) {
                var success = true;
                var response = new sdk.remotepay.RefundPaymentResponse();
                CloverConnector.populateBaseResponse(response, success, this.resultSuccessToResponseCode(success));
                response.setOrderId(refund.getOrderRef().getId());
                if (refund.getPayment()) {
                    response.setPaymentId(refund.getPayment().getId());
                }
                response.setRefund(refund);
                this.cloverConnector.broadcaster.notifyOnRefundPaymentResponse(response);
            }
            else {
                if (lastRefundResponse && lastRefundResponse.getRefund().getId() == refund.getId()) { //need to make sure it's the same refund before sending
                    this.cloverConnector.broadcaster.notifyOnRefundPaymentResponse(lastRefundResponse);
                }
                else {
                    var success = true;
                    var response = new sdk.remotepay.RefundPaymentResponse();
                    CloverConnector.populateBaseResponse(response, success, this.resultSuccessToResponseCode(success));
                    if (refund.getPayment()) {
                        response.setPaymentId(refund.getPayment().getId());
                    }
                    response.setRefund(refund);
                    this.cloverConnector.broadcaster.notifyOnRefundPaymentResponse(response);
                }
            }
        };
        // Weird mechanism to overload via TypeScript - https://blog.mariusschulz.com/2016/08/18/function-overloads-in-typescript
        InnerDeviceObserver.prototype.onFinishOk = function (paymentCreditOrRefund, signature, requestInfo) {
            if (paymentCreditOrRefund instanceof sdk.payments.Payment) {
                this.onFinishOkPayment(paymentCreditOrRefund, signature, requestInfo);
            }
            else if (paymentCreditOrRefund instanceof sdk.payments.Credit) {
                this.onFinishOkCredit(paymentCreditOrRefund);
            }
            else {
                this.onFinishOkRefund(paymentCreditOrRefund);
            }
        };
        InnerDeviceObserver.prototype.onFinishCancel_rmm = function (result, reason, message, requestInfo) {
            try {
                if (this.cloverConnector.device) {
                    this.cloverConnector.device.doShowWelcomeScreen();
                }
                if (requestInfo == TxTypeRequestInfo.PREAUTH_REQUEST) {
                    this.onFinishCancelPreAuth(result, reason, message);
                }
                else if (requestInfo == TxTypeRequestInfo.SALE_REQUEST) {
                    this.onFinishCancelSale(result, reason, message);
                }
                else if (requestInfo == TxTypeRequestInfo.AUTH_REQUEST) {
                    this.onFinishCancelAuth(result, reason, message);
                }
                else if (requestInfo == TxTypeRequestInfo.CREDIT_REQUEST) {
                    this.onFinishCancelManualRefund(result, reason, message);
                }
                else if (requestInfo == TxTypeRequestInfo.REFUND_REQUEST) {
                    if (this.lastPRR) {
                        this.cloverConnector.broadcaster.notifyOnRefundPaymentResponse(this.lastPRR);
                    }
                    else {
                        this.onFinishCancelRefund(result, reason, message);
                    }
                }
                else {
                    // Complete any un-resolved payment refund requests.
                    if (this.lastPRR) {
                        this.cloverConnector.broadcaster.notifyOnRefundPaymentResponse(this.lastPRR);
                        this.lastPRR = null;
                    }
                    else {
                        if (!requestInfo) {
                            this.logger.error('onFinishCancel called, requestInfo was null, and ' +
                                'could not determine the type of the message from the last request', arguments);
                        }
                        else {
                            this.logger.error('onFinishCancel called, but could not determine how to respond!', arguments);
                        }
                    }
                }
            }
            finally {
                // do nothing
            }
        };
        InnerDeviceObserver.prototype.onFinishCancel = function (requestInfo) {
            this.onFinishCancel_rmm(sdk.remotepay.ResponseCode.CANCEL, null, null, requestInfo);
        };
        InnerDeviceObserver.prototype.onFinishCancelPreAuth = function (result, reason, message) {
            var response = new sdk.remotepay.PreAuthResponse();
            CloverConnector.populateBaseResponse(response, false, result, reason ? reason : "Request Canceled", message ? message : "The PreAuth Request was canceled.");
            this.cloverConnector.broadcaster.notifyOnPreAuthResponse(response);
        };
        InnerDeviceObserver.prototype.onFinishCancelSale = function (result, reason, message) {
            var response = new sdk.remotepay.SaleResponse();
            CloverConnector.populateBaseResponse(response, false, result, reason ? reason : "Request Canceled", message ? message : "The Sale Request was canceled.");
            this.cloverConnector.broadcaster.notifyOnSaleResponse(response);
        };
        InnerDeviceObserver.prototype.onFinishCancelAuth = function (result, reason, message) {
            var response = new sdk.remotepay.AuthResponse();
            CloverConnector.populateBaseResponse(response, false, result, reason ? reason : "Request Canceled", message ? message : "The Auth Request was canceled.");
            this.cloverConnector.broadcaster.notifyOnAuthResponse(response);
        };
        InnerDeviceObserver.prototype.onFinishCancelManualRefund = function (result, reason, message) {
            var response = new sdk.remotepay.ManualRefundResponse();
            CloverConnector.populateBaseResponse(response, false, result, reason ? reason : "Request Canceled", message ? message : "The Manual Refund Request was canceled.");
            this.cloverConnector.broadcaster.notifyOnManualRefundResponse(response);
        };
        InnerDeviceObserver.prototype.onFinishCancelRefund = function (result, reason, message) {
            var response = new sdk.remotepay.RefundPaymentResponse();
            CloverConnector.populateBaseResponse(response, false, result, reason ? reason : "Request Canceled", message ? message : "The Refund Request was canceled.");
            this.cloverConnector.broadcaster.notifyOnRefundPaymentResponse(response);
        };
        InnerDeviceObserver.prototype.onVerifySignature = function (payment, signature) {
            var request = new sdk.remotepay.VerifySignatureRequest();
            request.setSignature(signature);
            request.setPayment(payment);
            this.cloverConnector.broadcaster.notifyOnVerifySignatureRequest(request);
        };
        InnerDeviceObserver.prototype.onConfirmPayment = function (payment, challenges) {
            var cpr = new sdk.remotepay.ConfirmPaymentRequest();
            cpr.setPayment(payment);
            cpr.setChallenges(challenges);
            this.cloverConnector.broadcaster.notifyOnConfirmPaymentRequest(cpr);
        };
        InnerDeviceObserver.prototype.onPaymentVoided = function (voidPaymentResponseMessage) {
            this.cloverConnector.showWelcomeScreen();
            var success = voidPaymentResponseMessage.getStatus() == sdk.remotemessage.ResultStatus.SUCCESS;
            var result = this.resultStatusToResponseCode(voidPaymentResponseMessage.getStatus());
            var response = new sdk.remotepay.VoidPaymentResponse();
            var payment = voidPaymentResponseMessage.getPayment();
            response.setPayment(payment);
            response.setPaymentId(payment != null ? payment.getId() : null);
            CloverConnector.populateBaseResponse(response, success, result, voidPaymentResponseMessage.getReason(), voidPaymentResponseMessage.getMessage());
            this.cloverConnector.broadcaster.notifyOnVoidPaymentResponse(response);
        };
        InnerDeviceObserver.prototype.onPaymentVoided_responseCode = function (code, reason, message) {
            var success = (code == sdk.remotepay.ResponseCode.SUCCESS);
            var response = new sdk.remotepay.VoidPaymentResponse();
            CloverConnector.populateBaseResponse(response, success, code, reason, message);
            this.cloverConnector.broadcaster.notifyOnVoidPaymentResponse(response);
        };
        InnerDeviceObserver.prototype.onPaymentRefundVoided = function (refund, code, reason, message) {
            var response = new sdk.remotepay.VoidPaymentRefundResponse();
            response.setSuccess(code == sdk.remotepay.ResponseCode.SUCCESS);
            response.setReason(reason != null ? reason : code.toString());
            response.setMessage(message != null ? message : "No extended information provided.");
            response.setRefund(refund);
            this.cloverConnector.broadcaster.notifyOnVoidPaymentRefundResponse(response);
        };
        InnerDeviceObserver.prototype.onKeyPressed = function (keyPress) {
            //TODO: For future use
        };
        InnerDeviceObserver.prototype.onPaymentRefundResponse = function (refundResponseMessage) {
            // hold the response for finishOk for the refund. See comments in onFinishOk(Refund)
            var success = refundResponseMessage.getCode() == sdk.remotemessage.TxState.SUCCESS;
            var response = new sdk.remotepay.RefundPaymentResponse();
            var reason = refundResponseMessage.getReason();
            CloverConnector.populateBaseResponse(response, success, this.resultSuccessToResponseCode(success), reason, refundResponseMessage.getMessage());
            response.setOrderId(refundResponseMessage.getOrderId());
            response.setPaymentId(refundResponseMessage.getPaymentId());
            response.setRefund(refundResponseMessage.getRefund());
            // NOTE: While this is currently needed, we are attempting to move away from this requirement
            this.lastPRR = response; // set this so we have the appropriate information for when onFinish(Refund) is called
        };
        InnerDeviceObserver.prototype.onPaymentRefundVoidResponse = function (voidPaymentRefundResponseMessage) {
            var success = (sdk.remotemessage.ResultStatus.SUCCESS == status);
            var response = new sdk.remotepay.VoidPaymentRefundResponse();
            var reason = voidPaymentRefundResponseMessage.getReason();
            CloverConnector.populateBaseResponse(response, success, this.resultSuccessToResponseCode(success), reason, voidPaymentRefundResponseMessage.getMessage());
            response.setRefundId(voidPaymentRefundResponseMessage.getRefund() ? voidPaymentRefundResponseMessage.getRefund().getId() : null);
            response.setReason(voidPaymentRefundResponseMessage.getReason());
        };
        InnerDeviceObserver.prototype.onVaultCardResponse = function (vaultCardResponseMessage, code, message) {
            var success = (sdk.remotemessage.ResultStatus.SUCCESS == vaultCardResponseMessage.getStatus());
            this.cloverConnector.device.doShowWelcomeScreen();
            var response = new sdk.remotepay.VaultCardResponse();
            response.setCard(vaultCardResponseMessage.getCard());
            CloverConnector.populateBaseResponse(response, success, code || this.resultSuccessToResponseCode(success), vaultCardResponseMessage.getReason(), message);
            this.cloverConnector.broadcaster.notifyOnVaultCardRespose(response);
        };
        InnerDeviceObserver.prototype.onCapturePreAuthError = function (result, reason, message, paymentId) {
            var response = new sdk.remotepay.CapturePreAuthResponse();
            CloverConnector.populateBaseResponse(response, false, result, reason, message);
            response.setPaymentId(paymentId);
            response.setTipAmount(0);
            this.cloverConnector.broadcaster.notifyOnTipAdjustAuthResponse(response);
        };
        InnerDeviceObserver.prototype.onCapturePreAuth = function (capturePreAuthResponseMessage) {
            var success = (sdk.remotemessage.ResultStatus.SUCCESS == capturePreAuthResponseMessage.getStatus());
            var response = new sdk.remotepay.CapturePreAuthResponse();
            CloverConnector.populateBaseResponse(response, success, this.resultStatusToResponseCode(capturePreAuthResponseMessage.getStatus()), capturePreAuthResponseMessage.getReason(), capturePreAuthResponseMessage.getMessage());
            response.setPaymentId(capturePreAuthResponseMessage.getPaymentId());
            response.setAmount(capturePreAuthResponseMessage.getAmount());
            response.setTipAmount(capturePreAuthResponseMessage.getTipAmount());
            this.cloverConnector.broadcaster.notifyOnCapturePreAuth(response);
        };
        InnerDeviceObserver.prototype.onIncrementPreAuthError = function (result, reason, message) {
            var response = new sdk.remotepay.IncrementPreAuthResponse();
            CloverConnector.populateBaseResponse(response, false, result, reason, message);
            this.cloverConnector.broadcaster.notifyOnIncrementPreAuthResponse(response);
        };
        InnerDeviceObserver.prototype.onIncrementPreAuthResponse = function (responseMsg) {
            var success = (sdk.remotemessage.ResultStatus.SUCCESS == responseMsg.getStatus());
            var response = new sdk.remotepay.IncrementPreAuthResponse();
            CloverConnector.populateBaseResponse(response, success, this.resultStatusToResponseCode(responseMsg.getStatus()), responseMsg.getReason(), null);
            response.setAuthorization(responseMsg.getAuthorization());
            this.cloverConnector.broadcaster.notifyOnIncrementPreAuthResponse(response);
        };
        InnerDeviceObserver.prototype.onCloseoutResponse = function (status, reason, batch) {
            var success = (status == sdk.remotemessage.ResultStatus.SUCCESS);
            var response = new sdk.remotepay.CloseoutResponse();
            CloverConnector.populateBaseResponse(response, success, this.resultSuccessToResponseCode(success), reason, null);
            response.setBatch(batch);
            this.cloverConnector.broadcaster.notifyCloseout(response);
        };
        InnerDeviceObserver.prototype.onDeviceDisconnected = function (device, message) {
            this.logger.debug('Disconnected ', message);
            this.cloverConnector.isReady = false;
            this.cloverConnector.broadcaster.notifyOnDisconnect(message);
        };
        InnerDeviceObserver.prototype.onDeviceConnected = function () {
            this.logger.debug('Connected');
            this.cloverConnector.isReady = false;
            this.cloverConnector.broadcaster.notifyOnConnect();
        };
        InnerDeviceObserver.prototype.onDeviceReady = function (device, drm) {
            this.logger.debug('Ready');
            this.cloverConnector.isReady = drm.getReady();
            // Build merchant info from the discoveryrequest
            var merchantInfo = new sdk.remotepay.MerchantInfo();
            merchantInfo.setMerchantID(drm.getMerchantId());
            merchantInfo.setMerchantMID(drm.getMerchantMId());
            merchantInfo.setMerchantName(drm.getMerchantName());
            var deviceInfo = new sdk.remotepay.DeviceInfo();
            merchantInfo.setDeviceInfo(deviceInfo);
            deviceInfo.setName(drm.getName());
            deviceInfo.setModel(drm.getModel());
            deviceInfo.setSerial(drm.getSerial());
            deviceInfo.setSupportsAcks(drm.getSupportsAcknowledgement());
            merchantInfo.setSupportsPreAuths(drm.getSupportsPreAuth());
            merchantInfo.setSupportsManualRefunds(drm.getSupportsManualRefund());
            merchantInfo.setSupportsTipAdjust(drm.getSupportsTipAdjust());
            merchantInfo.setSupportsAuths(drm.getSupportsAuth());
            merchantInfo.setSupportsVaultCards(drm.getSupportsVaultCard());
            this.cloverConnector.merchantInfo = merchantInfo;
            this.cloverConnector.device.setSupportsAck(drm.getSupportsAcknowledgement());
            this.cloverConnector.device.setSupportsVoidPaymentResponse(drm.getSupportsVoidPaymentResponse());
            if (drm.getReady()) {
                this.cloverConnector.broadcaster.notifyOnReady(merchantInfo);
            }
            else {
                this.cloverConnector.broadcaster.notifyOnConnect();
            }
        };
        InnerDeviceObserver.prototype.onDeviceError = function (errorEvent) {
            this.cloverConnector.broadcaster.notifyOnDeviceError(errorEvent);
        };
        InnerDeviceObserver.prototype.onPrintRefundPayment = function (payment, order, refund) {
            var response = new sdk.remotepay.PrintRefundPaymentReceiptResponse();
            response.setPayment(payment);
            response.setOrder(order);
            response.setRefund(refund);
            this.cloverConnector.broadcaster.notifyOnPrintRefundPaymentReceipt(response);
        };
        InnerDeviceObserver.prototype.onPrintMerchantReceipt = function (payment) {
            var message = new sdk.remotepay.PrintPaymentMerchantCopyReceiptMessage();
            message.setPayment(payment);
            this.cloverConnector.broadcaster.notifyOnPrintPaymentMerchantCopyReceipt(message);
        };
        InnerDeviceObserver.prototype.onPrintPaymentDecline = function (payment, reason) {
            var message = new sdk.remotepay.PrintPaymentDeclineReceiptMessage();
            message.setPayment(payment);
            message.setReason(reason);
            this.cloverConnector.broadcaster.notifyOnPrintPaymentDeclineReceipt(message);
        };
        InnerDeviceObserver.prototype.onPrintPayment = function (payment, order) {
            var message = new sdk.remotepay.PrintPaymentReceiptMessage();
            message.setPayment(payment);
            message.setOrder(order);
            this.cloverConnector.broadcaster.notifyOnPrintPaymentReceipt(message);
        };
        InnerDeviceObserver.prototype.onPrintCredit = function (credit) {
            var message = new sdk.remotepay.PrintManualRefundReceiptMessage();
            message.setCredit(credit);
            this.cloverConnector.broadcaster.notifyOnPrintCreditReceipt(message);
        };
        InnerDeviceObserver.prototype.onPrintCreditDecline = function (credit, reason) {
            var message = new sdk.remotepay.PrintManualRefundDeclineReceiptMessage();
            message.setCredit(credit);
            message.setReason(reason);
            this.cloverConnector.broadcaster.notifyOnPrintCreditDeclineReceipt(message);
        };
        InnerDeviceObserver.prototype.onMessageAck = function (messageId) {
            // TODO: for future use
        };
        InnerDeviceObserver.prototype.onPendingPaymentsResponse = function (success, pendingPayments, reason, message) {
            var result = success ? sdk.remotepay.ResponseCode.SUCCESS : sdk.remotepay.ResponseCode.ERROR;
            var response = new sdk.remotepay.RetrievePendingPaymentsResponse();
            CloverConnector.populateBaseResponse(response, success, result, reason, message);
            if (pendingPayments) {
                response.setPendingPaymentEntries(pendingPayments);
            }
            else {
                this.cloverConnector.device.doShowWelcomeScreen();
            }
            this.cloverConnector.broadcaster.notifyOnRetrievePendingPaymentResponse(response);
        };
        InnerDeviceObserver.prototype.onReadCardResponse = function (status, reason, cardData) {
            var success = (status == sdk.remotemessage.ResultStatus.SUCCESS);
            if (success) {
                var response = new sdk.remotepay.ReadCardDataResponse();
                CloverConnector.populateBaseResponse(response, success, this.resultSuccessToResponseCode(success), reason);
                response.setCardData(cardData);
                this.cloverConnector.device.doShowWelcomeScreen();
                this.cloverConnector.broadcaster.notifyOnReadCardDataResponse(response);
            }
            else if (status == sdk.remotemessage.ResultStatus.CANCEL) {
                this.onReadCardDataResponse(sdk.remotepay.ResponseCode.CANCEL, reason, '');
            }
            else {
                this.onReadCardDataResponse(sdk.remotepay.ResponseCode.FAIL, reason, '');
            }
        };
        InnerDeviceObserver.prototype.onMessageFromActivity = function (actionId, payload) {
            var message = new sdk.remotepay.MessageFromActivity();
            message.setAction(actionId);
            message.setPayload(payload);
            this.cloverConnector.broadcaster.notifyOnActivityMessage(message);
        };
        InnerDeviceObserver.prototype.onReadCardDataResponse = function (result, reason, message) {
            var success = (result == sdk.remotepay.ResponseCode.SUCCESS);
            this.cloverConnector.device.doShowWelcomeScreen();
            var response = new sdk.remotepay.ReadCardDataResponse();
            CloverConnector.populateBaseResponse(response, success, result, reason, message);
            this.cloverConnector.broadcaster.notifyOnReadCardDataResponse(response);
        };
        InnerDeviceObserver.prototype.onActivityResponse = function (status, payload, reason, actionId) {
            var success = (status == sdk.remotemessage.ResultStatus.SUCCESS);
            var result = success ? sdk.remotepay.ResponseCode.SUCCESS : sdk.remotepay.ResponseCode.CANCEL;
            var response = new sdk.remotepay.CustomActivityResponse();
            CloverConnector.populateBaseResponse(response, success, result, reason);
            response.setPayload(payload);
            response.setAction(actionId);
            this.cloverConnector.broadcaster.notifyOnActivityResponse(response);
        };
        InnerDeviceObserver.prototype.onDeviceStatusResponse = function (retrieveDeviceStatusResponseMessage) {
            var success = retrieveDeviceStatusResponseMessage.getStatus() ? retrieveDeviceStatusResponseMessage.getStatus() == sdk.remotemessage.ResultStatus.SUCCESS : true;
            var response = new sdk.remotepay.RetrieveDeviceStatusResponse();
            CloverConnector.populateBaseResponse(response, success, this.resultStatusToResponseCode(retrieveDeviceStatusResponseMessage.getStatus()), retrieveDeviceStatusResponseMessage.getReason());
            response.setState(retrieveDeviceStatusResponseMessage.getState());
            response.setData(retrieveDeviceStatusResponseMessage.getData());
            this.cloverConnector.broadcaster.notifyOnRetrieveDeviceStatusResponse(response);
        };
        InnerDeviceObserver.prototype.onInvalidStateTransitionResponse = function (status, reason, requestedTransition, state, data) {
            var success = (status == sdk.remotemessage.ResultStatus.SUCCESS);
            var response = new sdk.remotepay.InvalidStateTransitionResponse();
            var responseCode = this.resultStatusToResponseCode(status);
            CloverConnector.populateBaseResponse(response, success, responseCode, reason);
            response.setRequestedTransition(requestedTransition);
            response.setState(state);
            response.setData(data);
            this.cloverConnector.broadcaster.notifyOnInvalidStateTransitionResponse(response);
        };
        InnerDeviceObserver.prototype.onResetDeviceResponse = function (result, reason, state) {
            var success = (result == sdk.remotepay.ResponseCode.SUCCESS);
            var response = new sdk.remotepay.ResetDeviceResponse();
            CloverConnector.populateBaseResponse(response, success, result, reason);
            response.setState(state);
            this.cloverConnector.broadcaster.notifyOnResetDeviceResponse(response);
        };
        InnerDeviceObserver.prototype.onRetrievePaymentResponse = function (retrievePaymentResponseMessage) {
            var success = (retrievePaymentResponseMessage.getStatus() == sdk.remotemessage.ResultStatus.SUCCESS);
            var response = new sdk.remotepay.RetrievePaymentResponse();
            var responseCode = this.resultStatusToResponseCode(retrievePaymentResponseMessage.getStatus());
            CloverConnector.populateBaseResponse(response, success, responseCode, retrievePaymentResponseMessage.getReason(), retrievePaymentResponseMessage.getMessage());
            response.setExternalPaymentId(retrievePaymentResponseMessage.getExternalPaymentId());
            response.setQueryStatus(retrievePaymentResponseMessage.getQueryStatus());
            response.setPayment(retrievePaymentResponseMessage.getPayment());
            this.cloverConnector.broadcaster.notifyOnRetrievePaymentResponse(response);
        };
        InnerDeviceObserver.prototype.onRetrievePrintersResponse = function (result, printers) {
            var success = (result == sdk.remotepay.ResponseCode.SUCCESS);
            var response = new sdk.remotepay.RetrievePrintersResponse();
            CloverConnector.populateBaseResponse(response, success, result, null);
            response.setPrinters(printers);
            this.cloverConnector.broadcaster.notifyOnRetrievePrintersResponse(response);
        };
        InnerDeviceObserver.prototype.onPrintJobStatusResponse = function (result, printRequestId, printStatus) {
            var success = (result == sdk.remotepay.ResponseCode.SUCCESS);
            var response = new sdk.remotepay.PrintJobStatusResponse();
            CloverConnector.populateBaseResponse(response, success, result, printStatus);
            response.setStatus(printStatus);
            response.setPrintRequestId(printRequestId);
            this.cloverConnector.broadcaster.notifyOnPrintJobStatusResponse(response);
        };
        InnerDeviceObserver.prototype.onCustomerProvidedDataMessage = function (result, eventId, config, data) {
            var event = new sdk.remotepay.CustomerProvidedDataEvent();
            event.setResult(result);
            event.setSuccess(result === sdk.remotepay.ResponseCode.SUCCESS);
            event.setEventId(eventId);
            var dataProviderConfig = new sdk.remotepay.DataProviderConfig();
            dataProviderConfig.setType(config.getType());
            dataProviderConfig.setConfiguration(config.getConfiguration());
            event.setConfig(dataProviderConfig);
            event.setData(data);
            this.cloverConnector.broadcaster.notifyOnCustomerProvidedDataEvent(event);
        };
        InnerDeviceObserver.prototype.onDisplayReceiptOptionsResponse = function (status, reason) {
            var response = new sdk.remotepay.DisplayReceiptOptionsResponse();
            var responseCode = this.resultStatusToResponseCode(status);
            response.setReason(reason);
            response.setResult(responseCode);
            response.setSuccess(response.getResult() == sdk.remotepay.ResponseCode.SUCCESS);
            this.cloverConnector.broadcaster.notifyOnDisplayReceiptOptionsResponse(response);
        };
        InnerDeviceObserver.prototype.onSignatureCollected = function (signatureResponseMessage) {
            var response = new sdk.remotepay.SignatureResponse();
            var responseCode = this.resultStatusToResponseCode(signatureResponseMessage.getStatus());
            response.setReason(signatureResponseMessage.getReason());
            response.setResult(responseCode);
            response.setSuccess(response.getResult() == sdk.remotepay.ResponseCode.SUCCESS);
            response.setSignature(signatureResponseMessage.getSignature());
            this.cloverConnector.broadcaster.notifyOnSignatureCollected(response);
        };
        InnerDeviceObserver.prototype.onBalanceInquiryResponse = function (balanceInquiryResponseMessage) {
            var response = new sdk.remotepay.CheckBalanceResponse();
            var responseCode = this.resultStatusToResponseCode(balanceInquiryResponseMessage.getStatus());
            response.setReason(balanceInquiryResponseMessage.getReason());
            response.setResult(responseCode);
            response.setSuccess(response.getResult() == sdk.remotepay.ResponseCode.SUCCESS);
            var payment = balanceInquiryResponseMessage.getPayment();
            if (payment) {
                response.setAmount(payment.getAmount());
            }
            this.cloverConnector.broadcaster.notifyOnCheckBalanceResponse(response);
        };
        InnerDeviceObserver.prototype.onRequestTipResponse = function (requestTipResponseMessage) {
            var response = new sdk.remotepay.TipResponse();
            var responseCode = this.resultStatusToResponseCode(requestTipResponseMessage.getStatus());
            response.setReason(requestTipResponseMessage.getReason());
            response.setResult(responseCode);
            response.setSuccess(response.getResult() == sdk.remotepay.ResponseCode.SUCCESS);
            response.setTipAmount(requestTipResponseMessage.getAmount());
            this.cloverConnector.broadcaster.notifyOnTipResponse(response);
        };
        return InnerDeviceObserver;
    }());
    CloverConnector.InnerDeviceObserver = InnerDeviceObserver;
})(CloverConnector = exports.CloverConnector || (exports.CloverConnector = {}));
exports.CloverConnector = CloverConnector;

//# sourceMappingURL=../../../../maps/com/clover/remote/client/CloverConnector.js.map
