"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.WebSocketCloverDeviceConfiguration = void 0;
var DefaultCloverDevice_1 = require("./DefaultCloverDevice");
var Logger_1 = require("../util/Logger");
/**
 * The base for WebSocket device configurations.
 */
var WebSocketCloverDeviceConfiguration = /** @class */ (function () {
    /**
     * @param {string} applicationId - the applicationId that uniquely identifies the POS. e.g. com.company.MyPOS:2.3.1
     * @param {Object} webSocketFactoryFunction - the function that will return an instance of the CloverWebSocketInterface
     *    that will be used when connecting.
     * @param {IImageUtil} imageUtil - utility to translate images into base64 strings.
     * @param {number} heartbeatInterval - Frequency at which we will ping the device - millis. Defaults to -1, which means the heartbeat check is disabled.
     * @param {number} reconnectDelay - upon disconnect, duration to wait until a reconnect is attempted - millis. Defaults to 3000 (millis), minimum value is 3000 millis. To disable the reconnect set to -1.
     * @param {number} heartbeatDisconnectTimeout - If a response to a heartbeat ping is not received within this time we will call disconnect.
     */
    function WebSocketCloverDeviceConfiguration(applicationId, webSocketFactoryFunction, imageUtil, heartbeatInterval, reconnectDelay, heartbeatDisconnectTimeout) {
        this.logger = Logger_1.Logger.create();
        this.maxCharInMessage = 50000;
        this.imageUtil = imageUtil;
        this.appId = applicationId;
        this.webSocketImplClass = webSocketFactoryFunction;
        this.setHeartbeatInterval(heartbeatInterval);
        this.setReconnectDelay(reconnectDelay);
        this.setHeartbeatDisconnectTimeout(heartbeatDisconnectTimeout);
    }
    WebSocketCloverDeviceConfiguration.prototype.getApplicationId = function () {
        return this.appId;
    };
    WebSocketCloverDeviceConfiguration.prototype.getHeartbeatDisconnectTimeout = function () {
        return Math.max(WebSocketCloverDeviceConfiguration.DEFAULT_HEARTBEAT_DISCONNECT_TIMEOUT, this.heartbeatDisconnectTimeout);
    };
    WebSocketCloverDeviceConfiguration.prototype.setHeartbeatDisconnectTimeout = function (heartbeatDisconnectTimeout) {
        this.heartbeatDisconnectTimeout = heartbeatDisconnectTimeout || WebSocketCloverDeviceConfiguration.DEFAULT_HEARTBEAT_DISCONNECT_TIMEOUT;
    };
    WebSocketCloverDeviceConfiguration.prototype.getHeartbeatInterval = function () {
        if (this.heartbeatInterval === -1) {
            return this.heartbeatInterval;
        }
        return Math.max(WebSocketCloverDeviceConfiguration.MINIMUM_ALLOWED_HEARTBEAT_INTERVAL, this.heartbeatInterval);
    };
    WebSocketCloverDeviceConfiguration.prototype.setHeartbeatInterval = function (heartbeatInterval) {
        this.heartbeatInterval = heartbeatInterval || WebSocketCloverDeviceConfiguration.DEFAULT_HEARTBEAT_INTERVAL;
    };
    WebSocketCloverDeviceConfiguration.prototype.getReconnectDelay = function () {
        if (this.reconnectDelay === -1) {
            return this.reconnectDelay;
        }
        return Math.max(WebSocketCloverDeviceConfiguration.DEFAULT_RECONNECT_DELAY, this.reconnectDelay);
    };
    WebSocketCloverDeviceConfiguration.prototype.setReconnectDelay = function (reconnectDelay) {
        this.reconnectDelay = reconnectDelay || WebSocketCloverDeviceConfiguration.DEFAULT_RECONNECT_DELAY;
    };
    WebSocketCloverDeviceConfiguration.prototype.getCloverDeviceType = function () {
        return DefaultCloverDevice_1.DefaultCloverDevice;
    };
    WebSocketCloverDeviceConfiguration.prototype.getName = function () {
        return 'Clover WebSocket Connector';
    };
    /**
     * @override
     */
    WebSocketCloverDeviceConfiguration.prototype.getImageUtil = function () {
        return this.imageUtil;
    };
    WebSocketCloverDeviceConfiguration.prototype.getMaxMessageCharacters = function () {
        return this.maxCharInMessage;
    };
    WebSocketCloverDeviceConfiguration.DEFAULT_RECONNECT_DELAY = 3000; // millis - Also the minimum allowed.
    WebSocketCloverDeviceConfiguration.DEFAULT_HEARTBEAT_INTERVAL = -1; // disabled by default
    WebSocketCloverDeviceConfiguration.MINIMUM_ALLOWED_HEARTBEAT_INTERVAL = 2500; // millis - Do not allow pings more frequently than 2500.
    WebSocketCloverDeviceConfiguration.DEFAULT_HEARTBEAT_DISCONNECT_TIMEOUT = 3000; // millis - Also the minimum allowed.
    return WebSocketCloverDeviceConfiguration;
}());
exports.WebSocketCloverDeviceConfiguration = WebSocketCloverDeviceConfiguration;

//# sourceMappingURL=../../../../../maps/com/clover/remote/client/device/WebSocketCloverDeviceConfiguration.js.map
