"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CloverTransport = void 0;
/**
 * Clover Transport
 *
 * The clover transport facilitates notification distribution
 * from the device to a list of observers.
 */
var CloverTransport = /** @class */ (function () {
    function CloverTransport() {
        // List of observers to notify
        this.observers = [];
        // Flag to determine if the device is ready
        this.ready = false;
    }
    /**
     * @deprecated - see notifyConnected.
     */
    CloverTransport.prototype.notifyDeviceConnected = function () {
        this.notifyConnected();
    };
    /**
     * Notify observers that we are connected.  What "connected" means depends on the transport mechanism.
     *
     * For network (SNPD) this means that we have connected to the Clover device.
     * For cloud (CPD) this means that we have connected to the cloud proxy.
     */
    CloverTransport.prototype.notifyConnected = function () {
        var _this = this;
        this.observers.forEach(function (obs) {
            obs.onConnected(_this);
        });
    };
    /**
     * @deprecated - see notifyReady.
     */
    CloverTransport.prototype.notifyDeviceReady = function () {
        this.notifyReady();
    };
    /**
     * Notify observers that we are ready to send messages.  This has different meanings depending on the transport mechanism.
     *
     * For network (SNPD) this means that we have connected to and successfully pinged the Clover device.
     * For cloud (CPD) this means that we have connected to and successfully pinged the cloud proxy.
     *
     * This is generally used to indicate that we are clear to initiate the device via a Discovery Request.
     *
     * Note: this does not mean the device is ready to take a payment through the SDK, which is solely determined
     * by the receipt of a Discovery Response (see DefaultCloverDevice.notifyObserversReady).
     */
    CloverTransport.prototype.notifyReady = function () {
        var _this = this;
        this.ready = true;
        this.observers.forEach(function (obs) {
            obs.onReady(_this);
        });
    };
    /**
     * @deprecated - see notifyDisconnected.
     */
    CloverTransport.prototype.notifyDeviceDisconnected = function () {
        this.notifyDisconnected();
    };
    /**
     * Notify observers that we are disconnected.  What "disconnected" means depends on the transport mechanism.
     *
     * For network (SNPD) this means that we have disconnected from the Clover device.
     * For cloud (CPD) this means that we have disconnected from the cloud proxy.
     */
    CloverTransport.prototype.notifyDisconnected = function () {
        var _this = this;
        this.ready = false;
        this.observers.forEach(function (obs) {
            obs.onDisconnected(_this);
        });
    };
    CloverTransport.prototype.notifyConnectionAttemptComplete = function () {
        var _this = this;
        this.observers.forEach(function (obs) {
            obs.onConnectionAttemptComplete(_this);
        });
    };
    /**
     * Should be called by subclasses (_super.onMessage) when a message is received
     * in order to forward to all observers
     *
     * @param {string} message - The message we received
     */
    CloverTransport.prototype.onMessage = function (message) {
        this.observers.forEach(function (obs) {
            obs.onMessage(message);
        });
    };
    /**
     * Add new observer to receive notifications from the device
     *
     * @param {CloverTransportObserver} observer - the observer to notify
     */
    CloverTransport.prototype.subscribe = function (observer) {
        var _this = this;
        if (this.ready) {
            this.observers.forEach(function (obs) {
                obs.onReady(_this);
            });
        }
        this.observers.push(observer);
    };
    /**
     * Remove an observer from the list of observers
     *
     * @param {CloverTransportObserver} observer - the observer to remove
     */
    CloverTransport.prototype.unsubscribe = function (observer) {
        var indexOfObserver = this.observers.indexOf(observer);
        if (indexOfObserver !== -1) {
            this.observers.splice(indexOfObserver, 1);
        }
    };
    /**
     * Clear the observers list
     */
    CloverTransport.prototype.clearListeners = function () {
        this.observers.splice(0, this.observers.length);
    };
    CloverTransport.prototype.setObjectMessageSender = function (objectMessageSender) {
        this.objectMessageSender = objectMessageSender;
    };
    return CloverTransport;
}());
exports.CloverTransport = CloverTransport;

//# sourceMappingURL=../../../../../maps/com/clover/remote/client/transport/CloverTransport.js.map
